@@ -549,6 +549,7 @@ func (s *makeBootable20Suite) TestMakeSystemRunnableSealWithHookKeyProtector(c *
 		key, saveKey secboot.BootstrappedContainer,
 		primaryKey []byte,
 		volumesAuth *device.VolumesAuthOptions,
+		checkResult *secboot.PreinstallCheckResult,
 		model *asserts.Model,
 		modeenv *boot.Modeenv,
 		flags boot.MockSealKeyToModeenvFlags,
@@ -746,8 +747,9 @@ version: 5.0
 	myKey := secboot.CreateMockBootstrappedContainer()
 	myKey2 := secboot.CreateMockBootstrappedContainer()
 	chosenPrimaryKey := []byte("primarykey!")
-	volumesAuth := &device.VolumesAuthOptions{Mode: device.AuthModePassphrase, Passphrase: "test"}
-	obs.SetEncryptionParams(myKey, myKey2, chosenPrimaryKey, volumesAuth)
+	myVolumesAuth := &device.VolumesAuthOptions{Mode: device.AuthModePassphrase, Passphrase: "test"}
+	myCheckResult := &secboot.PreinstallCheckResult{}
+	obs.SetEncryptionParams(myKey, myKey2, chosenPrimaryKey, myVolumesAuth, myCheckResult)
 
 	// set a mock recovery kernel
 	readSystemEssentialCalls := 0
@@ -763,13 +765,14 @@ version: 5.0
 	defer restore()
 
 	sealKeyForBootChainsCalled := 0
-	restore = boot.MockSealKeyForBootChains(func(method device.SealingMethod, key, saveKey secboot.BootstrappedContainer, primaryKey []byte, volumesAuth *device.VolumesAuthOptions, params *boot.SealKeyForBootChainsParams) error {
+	restore = boot.MockSealKeyForBootChains(func(method device.SealingMethod, key, saveKey secboot.BootstrappedContainer, primaryKey []byte, volumesAuth *device.VolumesAuthOptions, checkResult *secboot.PreinstallCheckResult, params *boot.SealKeyForBootChainsParams) error {
 		sealKeyForBootChainsCalled++
 		c.Check(method, Equals, device.SealingMethodTPM)
 		c.Check(key, Equals, myKey)
 		c.Check(saveKey, Equals, myKey2)
 		c.Check(primaryKey, DeepEquals, chosenPrimaryKey)
-		c.Check(volumesAuth, Equals, volumesAuth)
+		c.Check(volumesAuth, Equals, myVolumesAuth)
+		c.Check(checkResult, Equals, myCheckResult)
 
 		recoveryBootLoader, hasRecovery := params.RoleToBlName[bootloader.RoleRecovery]
 		c.Assert(hasRecovery, Equals, true)
@@ -1253,7 +1256,7 @@ version: 5.0
 	myKey := secboot.CreateMockBootstrappedContainer()
 	myKey2 := secboot.CreateMockBootstrappedContainer()
 	chosenPrimaryKey := []byte("primarykey!")
-	obs.SetEncryptionParams(myKey, myKey2, chosenPrimaryKey, nil)
+	obs.SetEncryptionParams(myKey, myKey2, chosenPrimaryKey, nil, nil)
 
 	// set a mock recovery kernel
 	readSystemEssentialCalls := 0
@@ -1264,7 +1267,7 @@ version: 5.0
 	defer restore()
 
 	sealKeyForBootChainsCalled := 0
-	restore = boot.MockSealKeyForBootChains(func(method device.SealingMethod, key, saveKey secboot.BootstrappedContainer, primaryKey []byte, volumesAuth *device.VolumesAuthOptions, params *boot.SealKeyForBootChainsParams) error {
+	restore = boot.MockSealKeyForBootChains(func(method device.SealingMethod, key, saveKey secboot.BootstrappedContainer, primaryKey []byte, volumesAuth *device.VolumesAuthOptions, checkResult *secboot.PreinstallCheckResult, params *boot.SealKeyForBootChainsParams) error {
 		sealKeyForBootChainsCalled++
 		c.Check(method, Equals, device.SealingMethodTPM)
 		c.Check(key, Equals, myKey)
@@ -1448,7 +1451,7 @@ version: 5.0
 	myKey := secboot.CreateMockBootstrappedContainer()
 	myKey2 := secboot.CreateMockBootstrappedContainer()
 	chosenPrimaryKey := []byte("primarykey!")
-	obs.SetEncryptionParams(myKey, myKey2, chosenPrimaryKey, nil)
+	obs.SetEncryptionParams(myKey, myKey2, chosenPrimaryKey, nil, nil)
 
 	// set a mock recovery kernel
 	readSystemEssentialCalls := 0
@@ -1459,7 +1462,7 @@ version: 5.0
 	defer restore()
 
 	sealKeyForBootChainsCalled := 0
-	restore = boot.MockSealKeyForBootChains(func(method device.SealingMethod, key, saveKey secboot.BootstrappedContainer, primaryKey []byte, volumesAuth *device.VolumesAuthOptions, params *boot.SealKeyForBootChainsParams) error {
+	restore = boot.MockSealKeyForBootChains(func(method device.SealingMethod, key, saveKey secboot.BootstrappedContainer, primaryKey []byte, volumesAuth *device.VolumesAuthOptions, checkResult *secboot.PreinstallCheckResult, params *boot.SealKeyForBootChainsParams) error {
 		sealKeyForBootChainsCalled++
 		c.Check(method, Equals, device.SealingMethodTPM)
 		c.Check(key, DeepEquals, myKey)
