@@ -95,7 +95,7 @@ var u2fDevices = []u2fDevice{
 	{
 		Name:             "Thetis Key",
 		VendorIDPattern:  "1ea8",
-		ProductIDPattern: "f025|f825|fc26",
+		ProductIDPattern: "f025|f825|fc26|f829",
 	},
 	{
 		Name:             "Nitrokey FIDO U2F",
@@ -223,7 +223,7 @@ const u2fDevicesConnectedPlugAppArmor = `
 # Description: Allow write access to u2f hidraw devices.
 
 # Use a glob rule and rely on device cgroup for mediation.
-/dev/hidraw* rw,
+/dev/hidraw* rwk,
 
 # char 234-254 are used for dynamic assignment, which u2f devices are
 /run/udev/data/c23[4-9]:* r,
