@@ -168,7 +168,7 @@ func (s *deviceMgrGadgetSuite) mockModeenvForMode(c *C, mode string) {
 }
 
 func (s *deviceMgrGadgetSuite) setupModelWithGadget(c *C, gadget string) {
-	s.makeModelAssertionInState(c, "canonical", "pc-model", map[string]interface{}{
+	s.makeModelAssertionInState(c, "canonical", "pc-model", map[string]any{
 		"architecture": "amd64",
 		"kernel":       "pc-kernel",
 		"gadget":       gadget,
@@ -182,19 +182,19 @@ func (s *deviceMgrGadgetSuite) setupModelWithGadget(c *C, gadget string) {
 }
 
 func (s *deviceMgrGadgetSuite) setupUC20ModelWithGadget(c *C, gadget, grade string) {
-	s.makeModelAssertionInState(c, "canonical", "pc20-model", map[string]interface{}{
+	s.makeModelAssertionInState(c, "canonical", "pc20-model", map[string]any{
 		"display-name": "UC20 pc model",
 		"architecture": "amd64",
 		"base":         "core20",
 		"grade":        grade,
-		"snaps": []interface{}{
-			map[string]interface{}{
+		"snaps": []any{
+			map[string]any{
 				"name":            "pc-kernel",
 				"id":              "pckernelidididididididididididid",
 				"type":            "kernel",
 				"default-channel": "20",
 			},
-			map[string]interface{}{
+			map[string]any{
 				"name":            gadget,
 				"id":              "pcididididididididididididididid",
 				"type":            "gadget",
@@ -215,18 +215,18 @@ func (s *deviceMgrGadgetSuite) setupClassicWithModesModel(c *C, gadget string) *
 		Serial: "didididi",
 	})
 	return s.makeModelAssertionInState(c, "canonical", "classic-with-modes",
-		map[string]interface{}{
+		map[string]any{
 			"architecture": "amd64",
 			"classic":      "true",
 			"distribution": "ubuntu",
 			"base":         "core22",
-			"snaps": []interface{}{
-				map[string]interface{}{
+			"snaps": []any{
+				map[string]any{
 					"name": "pc-linux",
 					"id":   "pclinuxdidididididididididididid",
 					"type": "kernel",
 				},
-				map[string]interface{}{
+				map[string]any{
 					"name": gadget,
 					"id":   "pcididididididididididididididid",
 					"type": "gadget",
@@ -294,11 +294,19 @@ func (s *deviceMgrGadgetSuite) testUpdateGadgetSimple(c *C, grade string, encryp
 	var updateCalled bool
 	var passedRollbackDir string
 
+	defer boot.MockSetEfiBootVariables(func(description string, assetPath string, optionalData []byte) error {
+		c.Check(description, Equals, "ubuntu-test")
+		return nil
+	})()
+
 	if grade != "" {
 		bootDir := c.MkDir()
-		tbl := bootloadertest.Mock("trusted", bootDir).WithTrustedAssets()
+		tbl := bootloadertest.Mock("trusted", bootDir).WithTrustedAssetsAndEfi()
 		tbl.TrustedAssetsMap = map[string]string{"trusted-asset": "trusted-asset"}
 		tbl.ManagedAssetsList = []string{"managed-asset"}
+		tbl.EfiLoadOptionDesc = "ubuntu-test"
+		tbl.EfiLoadOptionPath = "/some/path"
+		tbl.EfiLoadOptionData = nil
 		bootloader.Force(tbl)
 		defer func() { bootloader.Force(nil) }()
 	}
@@ -335,16 +343,9 @@ func (s *deviceMgrGadgetSuite) testUpdateGadgetSimple(c *C, grade string, encryp
 			// check that the behavior is correct
 			m, err := boot.ReadModeenv("")
 			c.Assert(err, IsNil)
-			if encryption {
-				// with encryption enabled, trusted asset would
-				// have been picked up by the the observer and
-				// added to modenv
-				c.Assert(m.CurrentTrustedRecoveryBootAssets, NotNil)
-				c.Check(m.CurrentTrustedRecoveryBootAssets["trusted-asset"], DeepEquals,
-					[]string{"88478d8afe6925b348b9cd00085f3535959fde7029a64d7841b031acc39415c690796757afab1852a9e09da913a0151b"})
-			} else {
-				c.Check(m.CurrentTrustedRecoveryBootAssets, HasLen, 0)
-			}
+			c.Assert(m.CurrentTrustedRecoveryBootAssets, NotNil)
+			c.Check(m.CurrentTrustedRecoveryBootAssets["trusted-asset"], DeepEquals,
+				[]string{"88478d8afe6925b348b9cd00085f3535959fde7029a64d7841b031acc39415c690796757afab1852a9e09da913a0151b"})
 		}
 		return nil
 	})
@@ -735,7 +736,7 @@ volumes:
 		{"content.img", "updated content"},
 	})
 
-	r := gadget.MockVolumeStructureToLocationMap(func(gd gadget.GadgetData, _ gadget.Model, _ map[string]*gadget.Volume) (map[string]map[int]gadget.StructureLocation, map[string]map[int]*gadget.OnDiskStructure, error) {
+	r := gadget.MockVolumeStructureToLocationMap(func(_ gadget.Model, oldVolumes, _ map[string]*gadget.Volume) (map[string]map[int]gadget.StructureLocation, map[string]map[int]*gadget.OnDiskStructure, error) {
 		return map[string]map[int]gadget.StructureLocation{
 				"pc": {
 					0: {
@@ -744,7 +745,7 @@ volumes:
 					},
 				},
 			}, map[string]map[int]*gadget.OnDiskStructure{
-				"pc": gadget.OnDiskStructsFromGadget(gd.Info.Volumes["pc"]),
+				"pc": gadget.OnDiskStructsFromGadget(oldVolumes["pc"]),
 			}, nil
 	})
 	defer r()
@@ -825,7 +826,7 @@ func (s *deviceMgrGadgetSuite) TestCurrentAndUpdateInfo(c *C) {
 		Type:     snap.TypeGadget,
 	}
 
-	model := s.brands.Model("canonical", "pc-model", map[string]interface{}{
+	model := s.brands.Model("canonical", "pc-model", map[string]any{
 		"architecture": "amd64",
 		"kernel":       "pc-kernel",
 		"gadget":       "foo-gadget",
@@ -1146,7 +1147,7 @@ func (s *deviceMgrGadgetSuite) TestUpdateGadgetOnCoreFromKernelRemodel(c *C) {
 	chg, t := s.makeMinimalKernelAssetsUpdateChange(c)
 	devicestate.SetBootOkRan(s.mgr, true)
 
-	newModel := s.brands.Model("canonical", "pc-model", map[string]interface{}{
+	newModel := s.brands.Model("canonical", "pc-model", map[string]any{
 		"architecture": "amd64",
 		"kernel":       "pc-kernel",
 		"gadget":       "foo-gadget",
@@ -1177,10 +1178,11 @@ func (s *deviceMgrGadgetSuite) TestUpdateGadgetOnCoreFromKernelRemodel(c *C) {
 }
 
 type testGadgetCommandlineUpdateOpts struct {
-	updated       bool
-	isClassic     bool
-	grade         string
-	cmdlineAppend string
+	updated                          bool
+	isClassic                        bool
+	grade                            string
+	extraSnapdKernelCmdlineFragments map[string]string
+	cmdlineAppend                    string
 	// This is the part of cmdlineAppend that is allowed by the gadget
 	allowedCmdline string
 	// and this is the not allowed part
@@ -1224,6 +1226,18 @@ func (s *deviceMgrGadgetSuite) testGadgetCommandlineUpdateRun(c *C, fromFiles, t
 		tsk.Set("dangerous-cmdline-append", opts.cmdlineAppendDanger)
 		argsAppended = true
 	}
+	// Set extra snapd kernel command line args as well
+	for fragmentID, fragment := range opts.extraSnapdKernelCmdlineFragments {
+		updated, err := devicestate.SetExtraSnapdKernelCommandLineFragment(s.state, devicestate.ExtraSnapdKernelCmdlineFragmentID(fragmentID), fragment)
+		c.Assert(err, IsNil)
+		c.Check(updated, Equals, true)
+	}
+	if len(opts.extraSnapdKernelCmdlineFragments) == 0 {
+		checkPendingExtraSnapdFragments(c, s.state, false)
+	} else {
+		checkPendingExtraSnapdFragments(c, s.state, true)
+	}
+
 	chg := s.state.NewChange("sample", "...")
 	chg.AddTask(tsk)
 	s.state.Unlock()
@@ -1616,6 +1630,57 @@ func (s *deviceMgrGadgetSuite) TestUpdateGadgetCommandlineWithNewAppendedArgsSig
 	}
 }
 
+func (s *deviceMgrGadgetSuite) TestUpdateGadgetCommandlineWithExtraSnapdArgs(c *C) {
+	// no command line arguments prior to the gadget update
+	bootloader.Force(s.managedbl)
+	s.state.Lock()
+	s.setupUC20ModelWithGadget(c, "pc", "dangerous")
+	s.mockModeenvForMode(c, "run")
+	devicestate.SetBootOkRan(s.mgr, true)
+	s.state.Set("seeded", true)
+
+	// mimic system state
+	m, err := boot.ReadModeenv("")
+	c.Assert(err, IsNil)
+	m.CurrentKernelCommandLines = []string{
+		"snapd_recovery_mode=run console=ttyS0 console=tty1 panic=-1 args from gadget",
+	}
+	c.Assert(m.Write(), IsNil)
+	err = s.managedbl.SetBootVars(map[string]string{
+		"snapd_extra_cmdline_args": "args from gadget",
+	})
+	c.Assert(err, IsNil)
+	s.managedbl.SetBootVarsCalls = 0
+
+	s.state.Unlock()
+
+	sameFiles := [][]string{
+		{"meta/gadget.yaml", gadgetYaml},
+		{"cmdline.extra", "args from gadget"},
+	}
+	// old and new gadget have the same command line arguments, nothing changes
+	opts := testGadgetCommandlineUpdateOpts{
+		updated:   true,
+		isClassic: false,
+		grade:     "dangerous",
+		extraSnapdKernelCmdlineFragments: map[string]string{
+			"xkb": "xkb-val",
+		},
+	}
+	s.testGadgetCommandlineUpdateRun(c, sameFiles, sameFiles, "", "Updated kernel command line", opts)
+
+	m, err = boot.ReadModeenv("")
+	c.Assert(err, IsNil)
+	c.Check([]string(m.CurrentKernelCommandLines), DeepEquals, []string{
+		"snapd_recovery_mode=run console=ttyS0 console=tty1 panic=-1 args from gadget",
+		`snapd_recovery_mode=run console=ttyS0 console=tty1 panic=-1 args from gadget xkb-val`,
+	})
+	c.Check(s.managedbl.SetBootVarsCalls, Equals, 1)
+	s.state.Lock()
+	checkPendingExtraSnapdFragments(c, s.state, false)
+	s.state.Unlock()
+}
+
 func (s *deviceMgrGadgetSuite) TestUpdateGadgetCommandlineDroppedArgs(c *C) {
 	// no command line arguments prior to the gadget up
 	s.state.Lock()
