@@ -35,6 +35,8 @@ import (
 	"github.com/snapcore/snapd/seed"
 	"github.com/snapcore/snapd/seed/seedwriter"
 	"github.com/snapcore/snapd/snap"
+	"github.com/snapcore/snapd/snap/naming"
+	"github.com/snapcore/snapd/snap/snapfile"
 	"github.com/snapcore/snapd/strutil"
 )
 
@@ -55,7 +57,7 @@ func checkSystemRequestConflict(st *state.State, systemLabel string) error {
 	// holding the state lock so there is no race against mark-seeded
 	// clearing recovery system; recovery system is not cleared when seeding
 	// fails
-	modeEnv, err := maybeReadModeenv()
+	modeEnv, err := boot.MaybeReadModeenv()
 	if err != nil {
 		return err
 	}
@@ -76,12 +78,12 @@ func checkSystemRequestConflict(st *state.State, systemLabel string) error {
 	return nil
 }
 
-func systemFromSeed(label string, current *currentSystem) (*System, error) {
-	_, sys, err := loadSeedAndSystem(label, current)
+func systemFromSeed(label string, current *currentSystem, defaultRecoverySystem *DefaultRecoverySystem) (*System, error) {
+	_, sys, err := loadSeedAndSystem(label, current, defaultRecoverySystem)
 	return sys, err
 }
 
-func loadSeedAndSystem(label string, current *currentSystem) (seed.Seed, *System, error) {
+func loadSeedAndSystem(label string, current *currentSystem, defaultRecoverySystem *DefaultRecoverySystem) (seed.Seed, *System, error) {
 	s, err := seedOpen(dirs.SnapSeedDir, label)
 	if err != nil {
 		return nil, nil, fmt.Errorf("cannot open: %v", err)
@@ -95,13 +97,30 @@ func loadSeedAndSystem(label string, current *currentSystem) (seed.Seed, *System
 	if err != nil {
 		return nil, nil, fmt.Errorf("cannot obtain brand: %v", err)
 	}
-	system := &System{
-		Current: false,
-		Label:   label,
-		Model:   model,
-		Brand:   brand,
-		Actions: defaultSystemActions,
+
+	var optionalContainers OptionalContainers
+	if copier, ok := s.(seed.Copier); ok {
+		oc, err := copier.OptionalContainers()
+		if err != nil {
+			return nil, nil, fmt.Errorf("cannot list optional containers: %v", err)
+		}
+		optionalContainers = OptionalContainers{
+			Snaps:      oc.Snaps,
+			Components: oc.Components,
+		}
+	} else {
+		logger.Debugf("seed %q does not support copying", label)
 	}
+
+	system := &System{
+		Current:            false,
+		Label:              label,
+		Model:              model,
+		Brand:              brand,
+		Actions:            defaultSystemActions,
+		OptionalContainers: optionalContainers,
+	}
+	system.DefaultRecoverySystem = defaultRecoverySystem.sameAs(system)
 	if current.sameAs(system) {
 		system.Current = true
 		system.Actions = current.actions
@@ -151,9 +170,6 @@ func currentSystemForMode(st *state.State, mode string) (*currentSystem, error)
 }
 
 func currentSeededSystem(st *state.State) (*seededSystem, error) {
-	st.Lock()
-	defer st.Unlock()
-
 	var whatseeded []seededSystem
 	if err := st.Get("seeded-systems", &whatseeded); err != nil {
 		return nil, err
@@ -168,7 +184,7 @@ func currentSeededSystem(st *state.State) (*seededSystem, error) {
 }
 
 func seededSystemFromModeenv() (*seededSystem, error) {
-	modeEnv, err := maybeReadModeenv()
+	modeEnv, err := boot.MaybeReadModeenv()
 	if err != nil {
 		return nil, err
 	}
@@ -179,7 +195,7 @@ func seededSystemFromModeenv() (*seededSystem, error) {
 		return nil, fmt.Errorf("internal error: recovery system is unset")
 	}
 
-	system, err := systemFromSeed(modeEnv.RecoverySystem, nil)
+	system, err := systemFromSeed(modeEnv.RecoverySystem, nil, nil)
 	if err != nil {
 		return nil, err
 	}
@@ -194,11 +210,186 @@ func seededSystemFromModeenv() (*seededSystem, error) {
 	return seededSys, nil
 }
 
-// getInfoFunc is expected to return for a given snap name a snap.Info for that
-// snap and whether the snap is present is present. The second bit is relevant
-// for non-essential snaps mentioned in the model, which if present and having
-// an 'optional' presence in the model, will be added to the recovery system.
-type getSnapInfoFunc func(name string) (info *snap.Info, snapIsPresent bool, err error)
+// infoGetter is an interface that helps us get information about snaps and
+// components that are being installed in a new recovery system.
+type infoGetter interface {
+	// SnapInfo is expected to return for a given snap name a snap.Info for that
+	// snap, a path on disk where the snap file can be found, and whether the
+	// snap is present. The last bit is relevant for non-essential snaps
+	// mentioned in the model, which if present and having an 'optional'
+	// presence in the model, will be added to the recovery system.
+	SnapInfo(st *state.State, name string) (info *snap.Info, path string, snapIsPresent bool, err error)
+	// ComponentInfo is expected to return for a given component ref a
+	// snap.ComponentInfo for that component, a path on disk where the component
+	// file can be found, and whether the component is present. The last bit is
+	// relevant for non-essential components mentioned in the model, which if
+	// present and having an 'optional' presence in the model, will be added to
+	// the recovery system.
+	ComponentInfo(st *state.State, cref naming.ComponentRef, snapInfo *snap.Info) (info *snap.ComponentInfo, path string, present bool, err error)
+}
+
+// setupInfoGetter is an infoGetter that uses a recoverySystemSetup to get
+// information about snaps and components that are being installed in a new
+// recovery system.
+type setupInfoGetter struct {
+	setup *recoverySystemSetup
+}
+
+func (ig *setupInfoGetter) ComponentInfo(st *state.State, cref naming.ComponentRef, snapInfo *snap.Info) (info *snap.ComponentInfo, path string, present bool, err error) {
+	// components will come from one of these places:
+	//   * passed into the task via a list of side infos (these would have
+	//     come from a user posting snaps via the API)
+	//   * have just been downloaded by a task in setup.ComponentSetupTasks
+	//   * already installed on the system
+
+	logger.Debugf("requested info for component %q being installed during remodel", cref)
+	for _, l := range ig.setup.LocalComponents {
+		if l.SideInfo.Component != cref {
+			continue
+		}
+
+		snapf, err := snapfile.Open(l.Path)
+		if err != nil {
+			return nil, "", false, err
+		}
+
+		info, err := snap.ReadComponentInfoFromContainer(snapf, snapInfo, l.SideInfo)
+		if err != nil {
+			return nil, "", false, err
+		}
+
+		return info, l.Path, true, nil
+	}
+
+	// in a remodel scenario, the components may need to be fetched and thus
+	// their content can be different from what we have already installed, so we
+	// should first check the download tasks before consulting snapstate
+	for _, tskID := range ig.setup.ComponentSetupTasks {
+		taskWithComponentSetup := st.Task(tskID)
+		compsup, snapsup, err := snapstate.TaskComponentSetup(taskWithComponentSetup)
+		if err != nil {
+			return nil, "", false, err
+		}
+		if compsup.CompSideInfo.Component != cref {
+			continue
+		}
+
+		mountFile := compsup.BlobPath(snapsup.InstanceName())
+
+		f, err := snapfile.Open(mountFile)
+		if err != nil {
+			return nil, "", false, err
+		}
+
+		info, err = snap.ReadComponentInfoFromContainer(f, snapInfo, compsup.CompSideInfo)
+		if err != nil {
+			return nil, "", false, err
+		}
+
+		return info, mountFile, true, nil
+	}
+
+	// either a remodel scenario, in which case the component is not among the
+	// ones being fetched, or just creating a recovery system, in which case we
+	// use the components that are already installed
+
+	var snapst snapstate.SnapState
+	if err := snapstate.Get(st, snapInfo.InstanceName(), &snapst); err != nil {
+		if errors.Is(err, state.ErrNoState) {
+			return nil, "", false, nil
+		}
+		return nil, "", false, err
+	}
+
+	info, err = snapst.CurrentComponentInfo(cref)
+	if err != nil {
+		if errors.Is(err, snapstate.ErrNoCurrent) {
+			return nil, "", false, nil
+		}
+		return nil, "", false, err
+	}
+
+	cpi := snap.MinimalComponentContainerPlaceInfo(
+		cref.ComponentName,
+		info.Revision,
+		snapInfo.InstanceName(),
+	)
+
+	return info, cpi.MountFile(), true, nil
+}
+
+func (ig *setupInfoGetter) SnapInfo(st *state.State, name string) (info *snap.Info, path string, present bool, err error) {
+	// snaps will come from one of these places:
+	//   * passed into the task via a list of side infos (these would have
+	//     come from a user posting snaps via the API)
+	//   * have just been downloaded by a task in setup.SnapSetupTasks
+	//   * already installed on the system
+
+	for _, l := range ig.setup.LocalSnaps {
+		if l.SideInfo.RealName != name {
+			continue
+		}
+
+		snapf, err := snapfile.Open(l.Path)
+		if err != nil {
+			return nil, "", false, err
+		}
+
+		info, err := snap.ReadInfoFromSnapFile(snapf, l.SideInfo)
+		if err != nil {
+			return nil, "", false, err
+		}
+
+		return info, l.Path, true, nil
+	}
+
+	// in a remodel scenario, the snaps may need to be fetched and thus
+	// their content can be different from what we have in already installed
+	// snaps, so we should first check the download tasks before consulting
+	// snapstate
+	logger.Debugf("requested info for snap %q being installed during remodel", name)
+	for _, tskID := range ig.setup.SnapSetupTasks {
+		taskWithSnapSetup := st.Task(tskID)
+		snapsup, err := snapstate.TaskSnapSetup(taskWithSnapSetup)
+		if err != nil {
+			return nil, "", false, err
+		}
+		if snapsup.SnapName() != name {
+			continue
+		}
+		// by the time this task runs, the file has already been
+		// downloaded and validated
+		snapFile, err := snapfile.Open(snapsup.BlobPath())
+		if err != nil {
+			return nil, "", false, err
+		}
+		info, err = snap.ReadInfoFromSnapFile(snapFile, snapsup.SideInfo)
+		if err != nil {
+			return nil, "", false, err
+		}
+
+		return info, info.MountFile(), true, nil
+	}
+
+	// either a remodel scenario, in which case the snap is not
+	// among the ones being fetched, or just creating a recovery
+	// system, in which case we use the snaps that are already
+	// installed
+
+	info, err = snapstate.CurrentInfo(st, name)
+	if err == nil {
+		hash, _, err := asserts.SnapFileSHA3_384(info.MountFile())
+		if err != nil {
+			return nil, "", true, fmt.Errorf("cannot compute SHA3 of snap file: %v", err)
+		}
+		info.Sha3_384 = hash
+		return info, info.MountFile(), true, nil
+	}
+	if _, ok := err.(*snap.NotInstalledError); !ok {
+		return nil, "", false, err
+	}
+	return nil, "", false, nil
+}
 
 // snapWriteObserveFunc is called with the recovery system directory and the
 // path to a snap file being written. The snap file may be written to a location
@@ -214,7 +405,14 @@ type snapWriteObserveFunc func(systemDir, where string) error
 // recovery system - some snaps may be in the recovery system directory while
 // others may be in the common snaps directory shared between multiple recovery
 // systems on ubuntu-seed.
-func createSystemForModelFromValidatedSnaps(model *asserts.Model, label string, db asserts.RODatabase, getInfo getSnapInfoFunc, observeWrite snapWriteObserveFunc) (dir string, err error) {
+func createSystemForModelFromValidatedSnaps(
+	st *state.State,
+	model *asserts.Model,
+	label string,
+	db asserts.RODatabase,
+	getInfo infoGetter,
+	observeWrite snapWriteObserveFunc,
+) (dir string, err error) {
 	if model.Grade() == asserts.ModelGradeUnset {
 		return "", fmt.Errorf("cannot create a system for pre-UC20 model")
 	}
@@ -230,6 +428,10 @@ func createSystemForModelFromValidatedSnaps(model *asserts.Model, label string,
 		// RW mount of ubuntu-seed
 		SeedDir: boot.InitramfsUbuntuSeedDir,
 		Label:   label,
+		// due to the way that temp files are handled in daemon, they do not
+		// have .snap or .comp extensions. this flag lets us ignore that
+		// requirement.
+		IgnoreOptionFileExtentions: true,
 	}
 	w, err := seedwriter.New(model, wOpts)
 	if err != nil {
@@ -239,57 +441,85 @@ func createSystemForModelFromValidatedSnaps(model *asserts.Model, label string,
 	optsSnaps := make([]*seedwriter.OptionsSnap, 0, len(model.RequiredWithEssentialSnaps()))
 	// collect all snaps that are present
 	modelSnaps := make(map[string]*snap.Info)
+	// mapping of snap names to map of component names to component infos.
+	modelComponents := make(map[string]map[string]*snap.ComponentInfo)
 
-	getModelSnap := func(name string, essential bool, nonEssentialPresence string) error {
+	getModelSnap := func(sn *asserts.ModelSnap, essential bool) error {
 		kind := "essential"
 		if !essential {
 			kind = "non-essential"
-			if nonEssentialPresence != "" {
-				kind = fmt.Sprintf("non-essential but %v", nonEssentialPresence)
+			if sn.Presence != "" {
+				kind = fmt.Sprintf("non-essential but %v", sn.Presence)
 			}
 		}
-		info, present, err := getInfo(name)
+		snapInfo, snapPath, present, err := getInfo.SnapInfo(st, sn.Name)
 		if err != nil {
 			return fmt.Errorf("cannot obtain %v snap information: %v", kind, err)
 		}
-		if !essential && !present && nonEssentialPresence == "optional" {
+		if !essential && !present && sn.Presence == "optional" {
 			// non-essential snap which is declared as optionally
 			// present in the model
 			return nil
 		}
 		// grab those
-		logger.Debugf("%v snap: %v", kind, name)
+		logger.Debugf("%v snap: %v", kind, sn.Name)
 		if !present {
-			return fmt.Errorf("internal error: %v snap %q not present", kind, name)
+			return fmt.Errorf("internal error: %v snap %q not present", kind, sn.Name)
 		}
-		if _, ok := modelSnaps[info.MountFile()]; ok {
+		if _, ok := modelSnaps[snapPath]; ok {
 			// we've already seen this snap
 			return nil
 		}
+
+		var comps []seedwriter.OptionsComponent
+		modelComponents[sn.Name] = make(map[string]*snap.ComponentInfo)
+		for compName, comp := range sn.Components {
+			cref := naming.NewComponentRef(sn.Name, compName)
+			compInfo, compPath, present, err := getInfo.ComponentInfo(st, cref, snapInfo)
+			if err != nil {
+				return fmt.Errorf("cannot obtain component %q information: %v", cref, err)
+			}
+
+			if !present {
+				if comp.Presence == "optional" {
+					continue
+				}
+				return fmt.Errorf("internal error: required component %q not present", cref)
+			}
+
+			// since everything here is done by path, we omit the component
+			// names. this is what the seedwriter code wants.
+			comps = append(comps, seedwriter.OptionsComponent{
+				Path: compPath,
+			})
+			modelComponents[sn.Name][compPath] = compInfo
+		}
+
 		// present locally
 		// TODO: for grade dangerous we could have a channel here which is not
 		//       the model channel, handle that here
 		optsSnaps = append(optsSnaps, &seedwriter.OptionsSnap{
-			Path: info.MountFile(),
+			Path:       snapPath,
+			Components: comps,
 		})
-		modelSnaps[info.MountFile()] = info
+		modelSnaps[snapPath] = snapInfo
 		return nil
 	}
 
 	for _, sn := range model.EssentialSnaps() {
 		const essential = true
-		if err := getModelSnap(sn.SnapName(), essential, ""); err != nil {
+		if err := getModelSnap(sn, essential); err != nil {
 			return "", err
 		}
 	}
 	// snapd is implicitly needed
 	const snapdIsEssential = true
-	if err := getModelSnap("snapd", snapdIsEssential, ""); err != nil {
+	if err := getModelSnap(&asserts.ModelSnap{Name: "snapd"}, snapdIsEssential); err != nil {
 		return "", err
 	}
 	for _, sn := range model.SnapsWithoutEssential() {
 		const essential = false
-		if err := getModelSnap(sn.SnapName(), essential, sn.Presence); err != nil {
+		if err := getModelSnap(sn, essential); err != nil {
 			return "", err
 		}
 	}
@@ -301,48 +531,86 @@ func createSystemForModelFromValidatedSnaps(model *asserts.Model, label string,
 		fromDB := func(ref *asserts.Ref) (asserts.Assertion, error) {
 			return ref.Resolve(db.Find)
 		}
-		return asserts.NewFetcher(db, fromDB, save)
+
+		seqFromDB := func(ref *asserts.AtSequence) (asserts.Assertion, error) {
+			if ref.Sequence <= 0 {
+				hdrs, err := asserts.HeadersFromSequenceKey(ref.Type, ref.SequenceKey)
+				if err != nil {
+					return nil, err
+				}
+				return db.FindSequence(ref.Type, hdrs, -1, -1)
+			}
+			return ref.Resolve(db.Find)
+		}
+
+		return asserts.NewSequenceFormingFetcher(db, fromDB, seqFromDB, save)
 	}
-	f, err := w.Start(db, newFetcher)
-	if err != nil {
+
+	sf := seedwriter.MakeSeedAssertionFetcher(newFetcher)
+	if err := w.Start(db, sf); err != nil {
 		return "", err
 	}
+
 	// past this point the system directory is present
 
+	// TODO:COMPS: take into account local components
 	localSnaps, err := w.LocalSnaps()
 	if err != nil {
-		return recoverySystemDir, err
+		return "", err
 	}
 
 	localARefs := make(map[*seedwriter.SeedSnap][]*asserts.Ref)
 	for _, sn := range localSnaps {
 		info, ok := modelSnaps[sn.Path]
 		if !ok {
-			return recoverySystemDir, fmt.Errorf("internal error: no snap info for %q", sn.Path)
+			return "", fmt.Errorf("internal error: no snap info for %q", sn.Path)
 		}
+
+		asserted := info.ID() != ""
+
 		// TODO: the side info derived here can be different from what
 		// we have in snap.Info, but getting it this way can be
 		// expensive as we need to compute the hash, try to find a
 		// better way
-		_, aRefs, err := seedwriter.DeriveSideInfo(sn.Path, model, f, db)
+		_, assertions, err := seedwriter.DeriveSideInfo(sn.Path, model, sf, db)
 		if err != nil {
 			if !errors.Is(err, &asserts.NotFoundError{}) {
-				return recoverySystemDir, err
-			} else if info.SnapID != "" {
-				// snap info from state must have come
-				// from the store, so it is unexpected
-				// if no assertions for it were found
-				return recoverySystemDir, fmt.Errorf("internal error: no assertions for asserted snap with ID: %v", info.SnapID)
+				return "", err
+			}
+
+			// snap info from state must have come from the store, so it is
+			// unexpected if no assertions for it were found
+			if asserted {
+				return "", fmt.Errorf("internal error: no assertions for asserted snap with ID: %v", info.SnapID)
 			}
 		}
-		if err := w.SetInfo(sn, info); err != nil {
-			return recoverySystemDir, err
+
+		seedComps := make(map[string]*seedwriter.SeedComponent, len(sn.Components))
+		for compPath, comp := range modelComponents[info.SnapName()] {
+			if asserted {
+				_, compAssertions, err := seedwriter.DeriveComponentSideInfo(compPath, comp, info, model, sf, db)
+				if err != nil {
+					return "", err
+				}
+
+				assertions = append(assertions, compAssertions...)
+			}
+
+			seedComps[comp.Component.ComponentName] = &seedwriter.SeedComponent{
+				ComponentRef: comp.Component,
+				Path:         compPath,
+				Info:         comp,
+			}
 		}
-		localARefs[sn] = aRefs
+
+		if err := w.SetInfo(sn, info, seedComps); err != nil {
+			return "", err
+		}
+		localARefs[sn] = assertions
 	}
 
 	if err := w.InfoDerived(); err != nil {
-		return recoverySystemDir, err
+		return "", err
 	}
 
 	retrieveAsserts := func(sn, _, _ *seedwriter.SeedSnap) ([]*asserts.Ref, error) {
@@ -353,7 +621,7 @@ func createSystemForModelFromValidatedSnaps(model *asserts.Model, label string,
 		// get the list of snaps we need in this iteration
 		toDownload, err := w.SnapsToDownload()
 		if err != nil {
-			return recoverySystemDir, err
+			return "", err
 		}
 		// which should be empty as all snaps should be accounted for
 		// already
@@ -362,12 +630,12 @@ func createSystemForModelFromValidatedSnaps(model *asserts.Model, label string,
 			for _, sn := range toDownload {
 				which = append(which, sn.SnapName())
 			}
-			return recoverySystemDir, fmt.Errorf("internal error: need to download snaps: %v", strings.Join(which, ", "))
+			return "", fmt.Errorf("internal error: need to download snaps: %v", strings.Join(which, ", "))
 		}
 
 		complete, err := w.Downloaded(retrieveAsserts)
 		if err != nil {
-			return recoverySystemDir, err
+			return "", err
 		}
 		if complete {
 			logger.Debugf("snap processing for creating %q complete", label)
@@ -381,7 +649,7 @@ func createSystemForModelFromValidatedSnaps(model *asserts.Model, label string,
 
 	unassertedSnaps, err := w.UnassertedSnaps()
 	if err != nil {
-		return recoverySystemDir, err
+		return "", err
 	}
 	if len(unassertedSnaps) > 0 {
 		locals := make([]string, len(unassertedSnaps))
@@ -409,15 +677,15 @@ func createSystemForModelFromValidatedSnaps(model *asserts.Model, label string,
 		return osutil.CopyFile(src, dst, 0)
 	}
 	if err := w.SeedSnaps(copySnap); err != nil {
-		return recoverySystemDir, err
+		return "", err
 	}
 	if err := w.WriteMeta(); err != nil {
-		return recoverySystemDir, err
+		return "", err
 	}
 
 	bootSnaps, err := w.BootSnaps()
 	if err != nil {
-		return recoverySystemDir, err
+		return "", err
 	}
 	bootWith := &boot.RecoverySystemBootableSet{}
 	for _, sn := range bootSnaps {
@@ -430,7 +698,7 @@ func createSystemForModelFromValidatedSnaps(model *asserts.Model, label string,
 		}
 	}
 	if err := boot.MakeRecoverySystemBootable(model, boot.InitramfsUbuntuSeedDir, recoverySystemDirInRootDir, bootWith); err != nil {
-		return recoverySystemDir, fmt.Errorf("cannot make candidate recovery system %q bootable: %v", label, err)
+		return "", fmt.Errorf("cannot make candidate recovery system %q bootable: %v", label, err)
 	}
 	logger.Noticef("created recovery system %q", label)
 
