@@ -1,7 +1,7 @@
 // -*- Mode: Go; indent-tabs-mode: t -*-
 
 /*
- * Copyright (C) 2018 Canonical Ltd
+ * Copyright (C) 2018-2024 Canonical Ltd
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License version 3 as
@@ -25,9 +25,12 @@ import (
 	"os"
 	"path/filepath"
 	"strings"
+	"time"
 
 	. "gopkg.in/check.v1"
 
+	"github.com/snapcore/snapd/asserts"
+	"github.com/snapcore/snapd/asserts/assertstest"
 	"github.com/snapcore/snapd/dirs"
 	"github.com/snapcore/snapd/interfaces"
 	"github.com/snapcore/snapd/interfaces/ifacetest"
@@ -45,6 +48,7 @@ import (
 )
 
 type helpersSuite struct {
+	testutil.BaseTest
 	st *state.State
 }
 
@@ -53,12 +57,30 @@ var _ = Suite(&helpersSuite{})
 func (s *helpersSuite) SetUpTest(c *C) {
 	s.st = state.New(nil)
 	dirs.SetRootDir(c.MkDir())
+
+	s.MockModel(c, nil)
 }
 
 func (s *helpersSuite) TearDownTest(c *C) {
 	dirs.SetRootDir("")
 }
 
+func (s *helpersSuite) MockModel(c *C, extraHeaders map[string]any) {
+	model := assertstest.FakeAssertion(map[string]any{
+		"type":         "model",
+		"authority-id": "my-brand",
+		"series":       "16",
+		"brand-id":     "my-brand",
+		"model":        "my-model",
+		"gadget":       "gadget",
+		"kernel":       "krnl",
+		"architecture": "amd64",
+		"timestamp":    time.Now().Format(time.RFC3339),
+	}, extraHeaders).(*asserts.Model)
+
+	s.AddCleanup(snapstatetest.MockDeviceModel(model))
+}
+
 func (s *helpersSuite) TestIdentityMapper(c *C) {
 	var m ifacestate.SnapMapper = &ifacestate.IdentityMapper{}
 
@@ -145,11 +167,11 @@ func (s *helpersSuite) TestMappingFunctions(c *C) {
 func (s *helpersSuite) TestGetConns(c *C) {
 	s.st.Lock()
 	defer s.st.Unlock()
-	s.st.Set("conns", map[string]interface{}{
-		"app:network core:network": map[string]interface{}{
+	s.st.Set("conns", map[string]any{
+		"app:network core:network": map[string]any{
 			"auto":      true,
 			"interface": "network",
-			"slot-static": map[string]interface{}{
+			"slot-static": map[string]any{
 				"number": int(78),
 			},
 		},
@@ -177,11 +199,11 @@ func (s *helpersSuite) TestSetConns(c *C) {
 
 	// This has upper-case data internally, see export_test.go
 	ifacestate.SetConns(s.st, ifacestate.UpperCaseConnState())
-	var conns map[string]interface{}
+	var conns map[string]any
 	err := s.st.Get("conns", &conns)
 	c.Assert(err, IsNil)
-	c.Assert(conns, DeepEquals, map[string]interface{}{
-		"app:network core:network": map[string]interface{}{
+	c.Assert(conns, DeepEquals, map[string]any{
+		"app:network core:network": map[string]any{
 			"auto":      true,
 			"interface": "network",
 		}})
@@ -227,18 +249,18 @@ func (s *helpersSuite) TestHotplugSlotInfo(c *C) {
 	defs["foo"] = &ifacestate.HotplugSlotInfo{
 		Name:        "foo",
 		Interface:   "iface",
-		StaticAttrs: map[string]interface{}{"attr": "value"},
+		StaticAttrs: map[string]any{"attr": "value"},
 		HotplugKey:  "key",
 	}
 	ifacestate.SetHotplugSlots(s.st, defs)
 
-	var data map[string]interface{}
+	var data map[string]any
 	c.Assert(s.st.Get("hotplug-slots", &data), IsNil)
-	c.Assert(data, DeepEquals, map[string]interface{}{
-		"foo": map[string]interface{}{
+	c.Assert(data, DeepEquals, map[string]any{
+		"foo": map[string]any{
 			"name":         "foo",
 			"interface":    "iface",
-			"static-attrs": map[string]interface{}{"attr": "value"},
+			"static-attrs": map[string]any{"attr": "value"},
 			"hotplug-key":  "key",
 			"hotplug-gone": false,
 		}})
@@ -255,20 +277,20 @@ func (s *helpersSuite) TestFindConnsForHotplugKey(c *C) {
 
 	// Set conns in the state and get them via GetConns to avoid having to
 	// know the internals of connState struct.
-	st.Set("conns", map[string]interface{}{
-		"snap1:plug1 core:slot1": map[string]interface{}{
+	st.Set("conns", map[string]any{
+		"snap1:plug1 core:slot1": map[string]any{
 			"interface":   "iface1",
 			"hotplug-key": "key1",
 		},
-		"snap1:plug2 core:slot2": map[string]interface{}{
+		"snap1:plug2 core:slot2": map[string]any{
 			"interface":   "iface2",
 			"hotplug-key": "key1",
 		},
-		"snap1:plug3 core:slot3": map[string]interface{}{
+		"snap1:plug3 core:slot3": map[string]any{
 			"interface":   "iface2",
 			"hotplug-key": "key2",
 		},
-		"snap2:plug1 core:slot1": map[string]interface{}{
+		"snap2:plug1 core:slot1": map[string]any{
 			"interface":   "iface2",
 			"hotplug-key": "key2",
 		},
@@ -393,9 +415,9 @@ apps:
 
 	// Pretend that security profiles are out of date and mock the
 	// function that writes the new system key with one always panics.
-	restore = ifacestate.MockProfilesNeedRegeneration(func() bool { return true })
+	restore = ifacestate.MockProfilesNeedRegeneration(func(m *ifacestate.InterfaceManager) bool { return true })
 	defer restore()
-	restore = ifacestate.MockWriteSystemKey(func() error { panic("should not attempt to write system key") })
+	restore = ifacestate.MockWriteSystemKey(func(extraData interfaces.SystemKeyExtraData) error { panic("should not attempt to write system key") })
 	defer restore()
 	// Put a fake system key in place, we just want to see that file being removed.
 	err := os.MkdirAll(filepath.Dir(dirs.SnapSystemKeyFile), 0755)
@@ -468,9 +490,9 @@ func (s *helpersSuite) TestProfileRegenerationSetupMany(c *C) {
 	mockSnaps(c, st)
 
 	// Pretend that security profiles are out of date.
-	restore = ifacestate.MockProfilesNeedRegeneration(func() bool { return true })
+	restore = ifacestate.MockProfilesNeedRegeneration(func(m *ifacestate.InterfaceManager) bool { return true })
 	defer restore()
-	restore = ifacestate.MockWriteSystemKey(func() error {
+	restore = ifacestate.MockWriteSystemKey(func(extraData interfaces.SystemKeyExtraData) error {
 		writeKey = true
 		return nil
 	})
@@ -516,9 +538,9 @@ func (s *helpersSuite) TestProfileRegenerationSetupManyFailsSystemKeyNotWritten(
 	mockSnaps(c, st)
 
 	// Pretend that security profiles are out of date.
-	restore = ifacestate.MockProfilesNeedRegeneration(func() bool { return true })
+	restore = ifacestate.MockProfilesNeedRegeneration(func(m *ifacestate.InterfaceManager) bool { return true })
 	defer restore()
-	restore = ifacestate.MockWriteSystemKey(func() error {
+	restore = ifacestate.MockWriteSystemKey(func(extraData interfaces.SystemKeyExtraData) error {
 		writeKey = true
 		return nil
 	})
@@ -668,14 +690,15 @@ func (s *helpersSuite) TestAddHotplugSlot(c *C) {
 	c.Check(stateSlots, HasLen, 0)
 
 	si := &snap.SideInfo{Revision: snap.R(1)}
-	coreInfo := snaptest.MockSnap(c, coreSnapYaml, si)
+	coreAppSet := ifacetest.MockInfoAndAppSet(c, coreSnapYaml, nil, si)
+	c.Assert(repo.AddAppSet(coreAppSet), IsNil)
 
 	slot := &snap.SlotInfo{
 		Name:       "slot",
 		Label:      "label",
-		Snap:       coreInfo,
+		Snap:       coreAppSet.Info(),
 		Interface:  "test",
-		Attrs:      map[string]interface{}{"foo": "bar"},
+		Attrs:      map[string]any{"foo": "bar"},
 		HotplugKey: "key",
 	}
 	c.Assert(ifacestate.AddHotplugSlot(s.st, repo, stateSlots, iface, slot), IsNil)
@@ -693,7 +716,7 @@ func (s *helpersSuite) TestAddHotplugSlot(c *C) {
 	c.Check(stateSlot, DeepEquals, &ifacestate.HotplugSlotInfo{
 		Name:        "slot",
 		Interface:   "test",
-		StaticAttrs: map[string]interface{}{"foo": "bar"},
+		StaticAttrs: map[string]any{"foo": "bar"},
 		HotplugKey:  "key",
 		HotplugGone: false})
 }
@@ -736,7 +759,7 @@ func (s *helpersSuite) TestDiscardLateBackendViaSnapstate(c *C) {
 	defer dirs.SetRootDir("")
 
 	// security profiles do not need regeneration when crating the manager
-	restore := ifacestate.MockProfilesNeedRegeneration(func() bool { return false })
+	restore := ifacestate.MockProfilesNeedRegeneration(func(m *ifacestate.InterfaceManager) bool { return false })
 	defer restore()
 
 	backend := &ifacetest.TestSecurityBackendDiscardingLate{
