@@ -981,18 +981,24 @@ func (s *installSuite) TestEncryptionSupportInfoWithTPM(c *C) {
 			"secured", "encrypted", "2.68", "2.68", false, ErrorNone,
 			install.EncryptionSupportInfo{
 				Available: true, Disabled: false,
-				StorageSafety:           asserts.StorageSafetyEncrypted,
-				Type:                    device.EncryptionTypeLUKS,
-				PassphraseAuthAvailable: true,
+				StorageSafety: asserts.StorageSafetyEncrypted,
+				Type:          device.EncryptionTypeLUKS,
+				// TODO:FDEM: passphrase support is temporarily disabled
+				// during install even with supported snapd versions.
+				// PassphraseAuthAvailable: true,
+				PassphraseAuthAvailable: false,
 			},
 			nil,
 		}, {
 			"secured", "encrypted", "2.69", "2.69", true, ErrorNone,
 			install.EncryptionSupportInfo{
 				Available: true, Disabled: false,
-				StorageSafety:           asserts.StorageSafetyEncrypted,
-				Type:                    device.EncryptionTypeLUKS,
-				PassphraseAuthAvailable: true,
+				StorageSafety: asserts.StorageSafetyEncrypted,
+				Type:          device.EncryptionTypeLUKS,
+				// TODO:FDEM: passphrase support is temporarily disabled
+				// during install even with supported snapd versions.
+				// PassphraseAuthAvailable: true,
+				PassphraseAuthAvailable: false,
 			},
 			preinstallCheckContext,
 		}, {
@@ -1014,6 +1020,22 @@ func (s *installSuite) TestEncryptionSupportInfoWithTPM(c *C) {
 			},
 			preinstallCheckContext,
 		},
+		// PIN support smoke test
+		{
+			"secured", "encrypted", "2.99", "2.99", true, ErrorNone,
+			install.EncryptionSupportInfo{
+				Available: true, Disabled: false,
+				StorageSafety: asserts.StorageSafetyEncrypted,
+				Type:          device.EncryptionTypeLUKS,
+				// TODO:FDEM: PIN and passphrase support is temporarily disabled
+				// during install even with supported snapd versions.
+				// PassphraseAuthAvailable: true,
+				// PINAuthAvailable:        true,
+				PassphraseAuthAvailable: false,
+				PINAuthAvailable:        false,
+			},
+			preinstallCheckContext,
+		},
 	}
 	for i, tc := range testCases {
 		mockModel := s.mockHelperForEncryptionAvailabilityCheck(c, tc.isSupportedUbuntuHybrid, tc.detectedErrors, ErrorNone, map[string]any{
@@ -1868,6 +1890,17 @@ func (s *installSuite) testPrepareEncryptedSystemData(c *C, useTokens, hasCheckR
 	})
 	defer restore()
 
+	restore = install.MockSecbootCheckResult(func(pcc *secboot.PreinstallCheckContext) (*secboot.PreinstallCheckResult, error) {
+		if hasCheckResult {
+			c.Assert(pcc, Equals, expectedCheckContext)
+			return &secboot.PreinstallCheckResult{}, nil
+		} else {
+			c.Assert(true, Equals, false, Commentf("unexpected call to secbootCheckResult"))
+			return nil, errors.New("test error")
+		}
+	})
+	defer restore()
+
 	// We are required to call ObserveExistingTrustedRecoveryAssets on trusted observers
 	err = to.ObserveExistingTrustedRecoveryAssets(boot.InitramfsUbuntuSeedDir)
 	c.Assert(err, IsNil)
