@@ -38,14 +38,16 @@ type (
 	ManagerBackend managerBackend
 
 	MinimalInstallInfo  = minimalInstallInfo
+	SnapUpdate          = update
 	InstallSnapInfo     = installSnapInfo
-	ByType              = byType
 	DirMigrationOptions = dirMigrationOptions
 	Migration           = migration
 
 	ReRefreshSetup = reRefreshSetup
 
 	TooSoonError = tooSoonError
+
+	Target = target
 )
 
 var ComponentSetupTask = componentSetupTask
@@ -70,10 +72,10 @@ func MockSnapReadInfo(mock func(name string, si *snap.SideInfo) (*snap.Info, err
 	return func() { snapReadInfo = old }
 }
 
-func MockReadComponentInfo(mock func(compMntDir string) (*snap.ComponentInfo, error)) (restore func()) {
-	old := readComponentInfo
-	readComponentInfo = mock
-	return func() { readComponentInfo = old }
+func MockReadComponentInfo(mock func(compMntDir string, snapInfo *snap.Info, csi *snap.ComponentSideInfo) (*snap.ComponentInfo, error)) (restore func()) {
+	old := readComponentInfoAt
+	readComponentInfoAt = mock
+	return func() { readComponentInfoAt = old }
 }
 
 func MockMountPollInterval(intv time.Duration) (restore func()) {
@@ -139,6 +141,8 @@ var (
 	ArrangeSnapToWaitForBaseIfPresent    = arrangeSnapToWaitForBaseIfPresent
 	ArrangeSnapTaskSetsLinkageAndRestart = arrangeSnapTaskSetsLinkageAndRestart
 	ReRefreshSummary                     = reRefreshSummary
+
+	MaybeFindTasksetForSnap = maybeFindTasksetForSnap
 )
 
 const (
@@ -167,6 +171,11 @@ var (
 	CheckDBusServiceConflicts = checkDBusServiceConflicts
 )
 
+// desktop-file-ids
+var (
+	CheckDesktopFileIDsConflicts = checkDesktopFileIDsConflicts
+)
+
 // readme files
 var (
 	WriteSnapReadme = writeSnapReadme
@@ -288,6 +297,14 @@ func MockHasActiveConnection(fn func(st *state.State, iface string) (bool, error
 	}
 }
 
+func MockOnRefreshInhibitionTimeout(fn func(chg *state.Change, snapName string) error) (restore func()) {
+	old := onRefreshInhibitionTimeout
+	onRefreshInhibitionTimeout = fn
+	return func() {
+		onRefreshInhibitionTimeout = old
+	}
+}
+
 // re-refresh related
 var (
 	RefreshedSnaps     = refreshedSnaps
@@ -315,15 +332,7 @@ func MockReRefreshRetryTimeout(d time.Duration) (restore func()) {
 	}
 }
 
-// aux store info
-var (
-	AuxStoreInfoFilename = auxStoreInfoFilename
-	RetrieveAuxStoreInfo = retrieveAuxStoreInfo
-	KeepAuxStoreInfo     = keepAuxStoreInfo
-	DiscardAuxStoreInfo  = discardAuxStoreInfo
-)
-
-type AuxStoreInfo = auxStoreInfo
+type DisabledServices = disabledServices
 
 // link, misc handlers
 var (
@@ -393,8 +402,8 @@ var (
 // autorefresh
 var (
 	InhibitRefresh                       = inhibitRefresh
-	MaxInhibition                        = maxInhibition
 	MaxDuration                          = maxDuration
+	MaxInhibitionDuration                = maxInhibitionDuration
 	MaybeAddRefreshInhibitNotice         = maybeAddRefreshInhibitNotice
 	MaybeAsyncPendingRefreshNotification = maybeAsyncPendingRefreshNotification
 )
@@ -443,6 +452,7 @@ var (
 	AutoRefreshPhase1          = autoRefreshPhase1
 	RefreshRetain              = refreshRetain
 	RefreshCheck               = refreshAppsCheck
+	AffectsRunningHooks        = affectsRunningHooks
 
 	ExcludeFromRefreshAppAwareness = excludeFromRefreshAppAwareness
 )
@@ -561,3 +571,41 @@ func SetRestoredMonitoring(snapmgr *SnapManager, value bool) {
 func SetPreseed(snapmgr *SnapManager, value bool) {
 	snapmgr.preseed = value
 }
+
+func SetStoreCacheCleanNext(snapmgr *SnapManager, when time.Time) {
+	snapmgr.ensureStoreCacheCleanNext = when
+}
+
+func GetStoreCacheCleanNext(snapmgr *SnapManager) time.Time {
+	return snapmgr.ensureStoreCacheCleanNext
+}
+
+func SplitEssentialUpdates(deviceCtx DeviceContext, updates []SnapUpdate) (essential, nonEssential []SnapUpdate) {
+	return splitEssentialUpdates(deviceCtx, updates)
+}
+
+func MockAffectedSnapsByAttr(value map[string]AffectedSnapsFunc) (restore func()) {
+	old := affectedSnapsByAttr
+	affectedSnapsByAttr = value
+	return func() {
+		affectedSnapsByAttr = old
+	}
+}
+
+func MockAffectedSnapsByKind(value map[string]AffectedSnapsFunc) (restore func()) {
+	old := affectedSnapsByKind
+	affectedSnapsByKind = value
+	return func() {
+		affectedSnapsByKind = old
+	}
+}
+
+// CustomInstallGoal allows us to define custom implementations of installGoal
+// to be used in tests.
+type CustomInstallGoal struct {
+	ToInstall func(context.Context, *state.State, Options) ([]Target, error)
+}
+
+func (c *CustomInstallGoal) toInstall(ctx context.Context, st *state.State, opts Options) ([]Target, error) {
+	return c.ToInstall(ctx, st, opts)
+}
