@@ -30,6 +30,7 @@ import (
 	"path/filepath"
 	"strings"
 
+	efilib "github.com/canonical/go-efilib"
 	"github.com/canonical/go-tpm2"
 	"github.com/canonical/go-tpm2/mu"
 	sb "github.com/snapcore/secboot"
@@ -815,11 +816,24 @@ func buildPCRProtectionProfile(modelParams []*SealKeyModelParams, checkResult *P
 
 	// all models have the same dbx data, get it from the first one
 	var dbUpdates []*sb_efi.SignatureDBUpdate
-	if len(modelParams) > 0 && len(modelParams[0].EFISignatureDbxUpdate) > 0 {
-		dbUpdates = append(dbUpdates, &sb_efi.SignatureDBUpdate{
-			Name: sb_efi.Dbx,
-			Data: modelParams[0].EFISignatureDbxUpdate,
-		})
+	if len(modelParams) > 0 {
+		for _, update := range modelParams[0].EFISignatureDbxUpdates {
+			var db efilib.VariableDescriptor
+			switch update.Database {
+			case KeyDatabasePK:
+				db = sb_efi.PK
+			case KeyDatabaseKEK:
+				db = sb_efi.KEK
+			case KeyDatabaseDB:
+				db = sb_efi.Db
+			case KeyDatabaseDBX:
+				db = sb_efi.Dbx
+			}
+			dbUpdates = append(dbUpdates, &sb_efi.SignatureDBUpdate{
+				Name: db,
+				Data: update.Payload,
+			})
+		}
 	}
 
 	// build PCR protection policy
@@ -884,10 +898,21 @@ func buildPCRProtectionProfileLegacy(modelParams []*SealKeyModelParams, allowIns
 	for _, mp := range modelParams {
 		var updateDB []*sb_efi.SignatureDBUpdate
 
-		if len(mp.EFISignatureDbxUpdate) > 0 {
+		for _, update := range mp.EFISignatureDbxUpdates {
+			var db efilib.VariableDescriptor
+			switch update.Database {
+			case KeyDatabasePK:
+				db = sb_efi.PK
+			case KeyDatabaseKEK:
+				db = sb_efi.KEK
+			case KeyDatabaseDB:
+				db = sb_efi.Db
+			case KeyDatabaseDBX:
+				db = sb_efi.Dbx
+			}
 			updateDB = append(updateDB, &sb_efi.SignatureDBUpdate{
-				Name: sb_efi.Dbx,
-				Data: mp.EFISignatureDbxUpdate,
+				Name: db,
+				Data: update.Payload,
 			})
 		}
 
