@@ -18,6 +18,7 @@
 package snap
 
 import (
+	"errors"
 	"fmt"
 	"os"
 	"path/filepath"
@@ -166,6 +167,22 @@ func (c *componentPlaceInfo) MountDescription() string {
 	return fmt.Sprintf("Mount unit for %s, revision %s", c.ContainerName(), c.compRevision)
 }
 
+// DmVerityFile returns the name of the dm-verity hash file computed by the snap name and the digest.
+// If the snap doesn't contain integrity data or contains integrity data but not of type
+// "dm-verity", this will return an error.
+// XXX: dm-verity for components is not supported yet.
+func (c *componentPlaceInfo) DmVerityFile() (string, error) {
+	return "", errors.New("dm-verity for components not supported.")
+}
+
+// DmVerityDigest returns the dm-verity digest of the integrity data associated with the snap.
+// If the snap doesn't contain integrity data or contains integrity data but not of type
+// "dm-verity", this will return an error.
+// XXX: dm-verity for components is not supported yet.
+func (c *componentPlaceInfo) DmVerityDigest() (string, error) {
+	return "", errors.New("dm-verity for components not supported.")
+}
+
 // ComponentLinkPath returns the path for the symlink for a component for a
 // given snap revision. Note that this function only uses the ContainerName
 // method on the ContainerPlaceInfo. If that changes, callers of this function
