@@ -28,6 +28,7 @@ import (
 	"github.com/snapcore/snapd/jsonutil/safejson"
 	"github.com/snapcore/snapd/snap"
 	"github.com/snapcore/snapd/snap/channel"
+	"github.com/snapcore/snapd/snap/integrity"
 	"github.com/snapcore/snapd/snap/naming"
 	"github.com/snapcore/snapd/strutil"
 )
@@ -58,6 +59,7 @@ type storeSnap struct {
 	Website       string              `json:"website"`
 	StoreURL      string              `json:"store-url"`
 	Resources     []storeResource     `json:"resources"`
+	IntegrityData []storeIntegrity    `json:"integrity"`
 
 	// TODO: not yet defined: channel map
 
@@ -130,6 +132,21 @@ type storeInfo struct {
 	SnapID     string                  `json:"snap-id"`
 }
 
+type storeIntegrity struct {
+	Type    string `json:"type"`
+	Version string `json:"version"`
+	HashAlg string `json:"hash-algorithm"`
+	// the store side stores these fields using uint max size. Keeping these
+	// sizes here for consistency. snapd's side uses uint64 for these parameters
+	// in other structs, since these are dm-verity parameters and the kernel
+	// doesn't enforce any size limit on them.
+	DataBlockSize uint          `json:"data-block-size"`
+	HashBlockSize uint          `json:"hash-block-size"`
+	Digest        string        `json:"digest"`
+	Salt          string        `json:"salt"`
+	Download      storeDownload `json:"download"`
+}
+
 func infoFromStoreInfo(si *storeInfo) (*snap.Info, error) {
 	if len(si.ChannelMap) == 0 {
 		// if a snap has no released revisions, it _could_ be returned
@@ -280,6 +297,9 @@ func copyNonZeroFrom(src, dst *storeSnap) {
 	if len(src.Resources) > 0 {
 		dst.Resources = src.Resources
 	}
+	if len(src.IntegrityData) > 0 {
+		dst.IntegrityData = src.IntegrityData
+	}
 }
 
 func infoFromStoreSnap(d *storeSnap) (*snap.Info, error) {
@@ -351,6 +371,10 @@ func infoFromStoreSnap(d *storeSnap) (*snap.Info, error) {
 
 	addCategories(info, d.Categories)
 
+	if err := addIntegrityData(info, d.IntegrityData); err != nil {
+		return nil, err
+	}
+
 	return info, nil
 }
 
@@ -393,6 +417,43 @@ func addComponents(info *snap.Info, resources []storeResource) {
 	}
 }
 
+func addIntegrityData(info *snap.Info, integrityData []storeIntegrity) error {
+	if len(integrityData) == 0 {
+		return nil
+	}
+
+	// TODO: Currently there can only be one entry of integrity data therefore it is the one selected.
+	// In the future, an entry will be selected from the list to be downloaded based on extra information
+	// gathered from the system (such as preferred algorithm, preferred block size etc.)
+	i := integrityData[0]
+
+	version, err := strconv.ParseUint(i.Version, 10, 32)
+	if err != nil {
+		return err
+	}
+
+	integrity := &snap.IntegrityDataInfo{
+		IntegrityDataParams: integrity.IntegrityDataParams{
+			Type:          i.Type,
+			Version:       uint(version),
+			HashAlg:       i.HashAlg,
+			DataBlockSize: uint64(i.DataBlockSize),
+			HashBlockSize: uint64(i.HashBlockSize),
+			Digest:        i.Digest,
+			Salt:          i.Salt,
+		},
+		DownloadInfo: snap.DownloadInfo{
+			DownloadURL: i.Download.URL,
+			Size:        i.Download.Size,
+			Sha3_384:    i.Download.Sha3_384,
+		},
+	}
+
+	info.IntegrityData = integrity
+
+	return nil
+}
+
 func downloadInfoFromStoreDownload(d storeDownload) snap.DownloadInfo {
 	downloadInfo := snap.DownloadInfo{
 		DownloadURL: d.URL,
