@@ -109,8 +109,8 @@ type Config struct {
 	FindFields  []string
 	DeltaFormat string
 
-	// CacheDownloads is the number of downloads that should be cached
-	CacheDownloads int
+	// CachePolicy defines the cache policy for downloaded snaps
+	CachePolicy CachePolicy
 
 	// Proxy returns the HTTP proxy to use when talking to the store
 	Proxy func(*http.Request) (*url.URL, error)
@@ -320,10 +320,10 @@ func init() {
 	if err != nil {
 		panic(err)
 	}
-	defaultConfig.DetailFields = jsonutil.StructFields((*snapDetails)(nil), "snap_yaml_raw")
-	defaultConfig.InfoFields = jsonutil.StructFields((*storeSnap)(nil), "snap-yaml")
+	defaultConfig.DetailFields = jsonutil.StructFields((*snapDetails)(nil), "snap_yaml_raw", "integrity")
+	defaultConfig.InfoFields = jsonutil.StructFields((*storeSnap)(nil), "snap-yaml", "integrity")
 	defaultConfig.FindFields = append(jsonutil.StructFields((*storeSnap)(nil),
-		"architectures", "created-at", "epoch", "name", "snap-id", "snap-yaml", "resources"),
+		"architectures", "created-at", "epoch", "name", "snap-id", "snap-yaml", "resources", "integrity"),
 		"channel")
 }
 
@@ -426,7 +426,7 @@ func New(cfg *Config, dauthCtx DeviceAndAuthContext) *Store {
 	}
 	store.auth = auth
 
-	store.SetCacheDownloads(cfg.CacheDownloads)
+	store.SetCachePolicy(cfg.CachePolicy)
 
 	return store
 }
