@@ -24,13 +24,21 @@ prepare_classic_rootfs() {
     [ -e "$DESTDIR"/dev/random ] || sudo mknod -m 666 "$DESTDIR"/dev/random c 1 8
     [ -e "$DESTDIR"/dev/urandom ] || sudo mknod -m 666 "$DESTDIR"/dev/urandom c 1 9
 
+    # and mount /proc which is needed by systemd.postinst script
+    [ -e "$DESTDIR"/proc/cmdline ] || sudo mount -t proc none "$DESTDIR"/proc
+
     if [ "$ROLE" = spread ]; then
-        # ensure resolving works inside the chroot
-        echo "nameserver 8.8.8.8" | sudo tee -a "$DESTDIR"/etc/resolv.conf
+    # ensure the proxy and resolv configuration is set inside the chroot
+        if [ "${SNAPD_USE_PROXY:-}" = true ]; then
+            cp /etc/resolv.conf "$DESTDIR"/etc/resolv.conf
+            cp /etc/environment "$DESTDIR"/etc/environment
+        else
+            echo "nameserver 8.8.8.8" | sudo tee -a "$DESTDIR"/etc/resolv.conf
+        fi
 
         # install additional packages
         sudo chroot "$DESTDIR" /usr/bin/sh -c "DEBIAN_FRONTEND=noninteractive apt update"
-        local pkgs="snapd ssh openssh-server sudo iproute2 iputils-ping isc-dhcp-client netplan.io vim-tiny kmod cloud-init cryptsetup"
+        local pkgs="snapd ssh openssh-server sudo iproute2 iputils-ping isc-dhcp-client netplan.io vim-tiny kmod cloud-init cryptsetup systemd-timesyncd"
         sudo chroot "$DESTDIR" /usr/bin/sh -c \
              "DEBIAN_FRONTEND=noninteractive apt install --no-install-recommends -y $pkgs"
         # netplan config
@@ -81,6 +89,9 @@ EOF
     sudo tee -a "$DESTDIR/etc/fstab" <<'EOF'
 /run/mnt/ubuntu-boot/EFI/ubuntu /boot/grub none bind
 EOF
+
+    # we no longer need the mounted /proc
+    sudo umount "$DESTDIR"/proc
 }
 
 # get target dir from user
@@ -116,7 +127,12 @@ else
             pointrel=
             ;;
     esac
-    wget -q -c http://cdimage.ubuntu.com/ubuntu-base/releases/"$release"/release/ubuntu-base-"$release""$pointrel"-base-amd64.tar.gz -O "$BASETAR"
+    if [ "$release" == "26.04" ]; then
+        # TODO: remove this workaround when the 26.04 rootfs is released, for now use the daily base
+        wget -q -c http://cdimage.ubuntu.com/ubuntu-base/daily/current/resolute-base-amd64.tar.gz -O "$BASETAR"
+    else
+        wget -q -c http://cdimage.ubuntu.com/ubuntu-base/releases/"$release"/release/ubuntu-base-"$release""$pointrel"-base-amd64.tar.gz -O "$BASETAR"
+    fi
     sudo tar -C "$DST" -xf "$BASETAR"
     ROLE=spread
 fi
