@@ -17,6 +17,7 @@
 : "${NESTED_SIGN_SNAPS_FAKESTORE:=false}"
 : "${NESTED_REPACK_FOR_FAKESTORE:=false}"
 : "${NESTED_FAKESTORE_SNAP_DECL_PC_GADGET:=}"
+: "${NESTED_FAKESTORE_SNAP_DECL_PC_KERNEL:=}"
 : "${NESTED_UBUNTU_IMAGE_SNAPPY_FORCE_SAS_URL:=}"
 : "${NESTED_UBUNTU_IMAGE_PRESEED_KEY:=}"
 : "${NESTED_UBUNTU_SEED_SIZE:=}"
@@ -754,7 +755,16 @@ nested_prepare_kernel() {
 
         # sign the pc-kernel snap with fakestore if requested
         if [ "$NESTED_SIGN_SNAPS_FAKESTORE" = "true" ]; then
-            "$TESTSTOOLS"/store-state make-snap-installable --noack "$NESTED_FAKESTORE_BLOB_DIR" "$(nested_get_extra_snaps_path)/$output_name" "$snap_id"
+            local extra_decl_args=""
+            local kernel_decl="$NESTED_FAKESTORE_SNAP_DECL_PC_KERNEL"
+            if [ -z "$kernel_decl" ] ; then
+                kernel_decl="$TESTSLIB/assertions/pc-kernel-snap-decl-extras.json"
+            fi
+            if [ -n "$kernel_decl" ]; then
+                extra_decl_args="--extra-decl-json $kernel_decl"
+            fi
+            # shellcheck disable=SC2086
+            "$TESTSTOOLS"/store-state make-snap-installable --noack $extra_decl_args "$NESTED_FAKESTORE_BLOB_DIR" "$(nested_get_extra_snaps_path)/$output_name" "$snap_id"
         fi
     fi
 }
@@ -1505,7 +1515,13 @@ nested_start_core_vm_unit() {
                 remote.exec "cloud-init status" || ret=$?
                 if [ "$ret" -ne 0 ] && [ "$ret" -ne 2 ]; then
                     echo "cloud-init finished with error $ret"
-                    exit 1
+                    # FIXME: remove core26 case.
+                    # See https://github.com/canonical/cloud-init/issues/6699
+                    if nested_is_core_26_system; then
+                        echo "Ignoring error on core26 for now"
+                    else
+                        exit 1
+                    fi
                 fi
             fi
         fi
