@@ -39,15 +39,19 @@ description: kernel component for testing purposes
 EOF
     # Replace _ or - with [_-], as it can be any of these
     glob_mod_name=$(printf '%s' "$mod_name" | sed -r 's/[-_]/[-_]/g')
-    module_path=$(find kernel -name "${glob_mod_name}.ko*")
+    # TODO: search only in kernel/modules to avoid duplicates (pc-kernel in
+    # 26/edge has both modules/ and lib/modules/). remove this hack once
+    # the kernel snap is fixed
+    module_path=$(find kernel/modules -name "${glob_mod_name}.ko*")
     cp "$module_path" "$comp_ko_dir"
     snap pack --filename="${kernel_name}+${comp_name}".comp "$comp_name"
 
     # Create kernel without the kernel module
     rm "$module_path"
-    # depmod wants a lib subdir
-    mkdir -p kernel/lib
-    ln -s ../modules kernel/lib/modules
+    if [ ! -e kernel/lib/modules ]; then
+        mkdir -p kernel/lib
+        ln -s ../modules kernel/lib/modules
+    fi
     depmod -b kernel/ "$kern_ver"
     rm "${kernel_snap_file}"
     # append component meta-information
