@@ -37,8 +37,6 @@ import (
 	"github.com/snapcore/snapd/timeout"
 )
 
-const GraphicalSessionTarget = "graphical-session.target"
-
 // SnapServicesUnitOptions is a struct for controlling the generated service
 // definition for a snap service.
 type SnapServicesUnitOptions struct {
@@ -146,18 +144,9 @@ Wants={{.PrerequisiteTarget}}
 {{- if .After}}
 After={{ stringsJoin .After " " }}
 {{- end}}
-{{- if .Requisite}}
-Requisite={{ stringsJoin .Requisite " " }}
-{{- end}}
 {{- if .Before}}
 Before={{ stringsJoin .Before " "}}
 {{- end}}
-{{- if .BindsTo}}
-BindsTo={{ stringsJoin .BindsTo " " }}
-{{- end}}
-{{- if .PartOf}}
-PartOf={{ stringsJoin .PartOf " " }}
-{{- end}}
 {{- if .CoreMountedSnapdSnapDep}}
 Wants={{ stringsJoin .CoreMountedSnapdSnapDep " "}}
 After={{ stringsJoin .CoreMountedSnapdSnapDep " "}}
@@ -296,9 +285,6 @@ WantedBy={{.ServicesTarget}}
 		Before                   []string
 		After                    []string
 		Requires                 []string
-		BindsTo                  []string
-		PartOf                   []string
-		Requisite                []string
 		InterfaceServiceSnippets string
 		InterfaceUnitSnippets    string
 		SliceUnit                string
@@ -333,11 +319,8 @@ WantedBy={{.ServicesTarget}}
 		BusName:           busName,
 		SuccessExitStatus: appInfo.SuccessExitStatus,
 
-		Before:    generateServiceNames(appInfo.Snap, appInfo.Before),
-		After:     generateServiceNames(appInfo.Snap, appInfo.After),
-		BindsTo:   generateServiceNames(appInfo.Snap, appInfo.BindsTo),
-		PartOf:    generateServiceNames(appInfo.Snap, appInfo.PartOf),
-		Requisite: generateServiceNames(appInfo.Snap, appInfo.Requisite),
+		Before: generateServiceNames(appInfo.Snap, appInfo.Before),
+		After:  generateServiceNames(appInfo.Snap, appInfo.After),
 
 		// systemd runs as PID 1 so %h will not work.
 		Home: "/root",
@@ -355,16 +338,6 @@ WantedBy={{.ServicesTarget}}
 		// FIXME: ideally use UserDataDir("%h"), but then the
 		// unit fails if the directory doesn't exist.
 		wrapperData.WorkingDir = appInfo.Snap.DataDir()
-		if appInfo.HasPlug("desktop") {
-			wrapperData.After = append(wrapperData.After, GraphicalSessionTarget)
-			if appInfo.Daemon == "dbus" || len(appInfo.Sockets) != 0 {
-				wrapperData.PartOf = append(wrapperData.PartOf, GraphicalSessionTarget)
-			} else {
-				wrapperData.BindsTo = append(wrapperData.BindsTo, GraphicalSessionTarget)
-			}
-			wrapperData.ServicesTarget = GraphicalSessionTarget
-			wrapperData.Requisite = append(wrapperData.Requisite, GraphicalSessionTarget)
-		}
 	default:
 		panic("unknown snap.DaemonScope")
 	}
